// Visual ClientView.cpp : implementation of the CVisualClientView class
//

#include "stdafx.h"
#include "Visual Client.h"

#include "Visual ClientDoc.h"
#include "Visual ClientView.h"

#include "ConnectDlg.h"
#include "TasksDlg.h"
#include "CompilersDlg.h"
#include "SendDlg.h"
#include ".\visual clientview.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

using namespace std;

// CVisualClientView

IMPLEMENT_DYNCREATE(CVisualClientView, CEditView)

BEGIN_MESSAGE_MAP(CVisualClientView, CEditView)
	ON_COMMAND(ID_ACTIONS_CONNECT, OnActionsConnect)
	ON_COMMAND(ID_ACTIONS_DISCONNECT, OnActionsDisconnect)
	ON_COMMAND(ID_ACTIONS_TASKS, OnActionsTasks)
	ON_COMMAND(ID_ACTIONS_COMPILERS, OnActionsCompilers)
	ON_COMMAND(ID_ACTIONS_SEND, OnActionsSend)
	ON_WM_CREATE()
	ON_COMMAND(ID_ACTIONS_LOG, OnActionsLog)
	ON_COMMAND(ID_ACTIONS_RATING, OnActionsRating)
END_MESSAGE_MAP()

// CVisualClientView construction/destruction

CVisualClientView::CVisualClientView()
: m_ptrClient(NULL)
, state(st_disconnected)
, m_ptrLogDlg(NULL)
, m_bRating(false)
{
	m_ptrClient = new CClient(this);
	m_font.CreatePointFont(100, "Courier New");

	m_ptrLogDlg = new CLogDlg();
	m_ptrLogDlg->Create(IDD_LOG_DIALOG, NULL);

	ZeroMemory(&m_pi, sizeof(m_pi));

	ZeroMemory(&m_si, sizeof(STARTUPINFO));
	m_si.lpReserved = NULL;
	m_si.lpDesktop = NULL;
	m_si.lpTitle = NULL;
	m_si.dwFlags = STARTF_USESHOWWINDOW;
	m_si.wShowWindow = SW_SHOWNORMAL;
	m_si.cbReserved2 = 0;
	m_si.lpReserved2 = NULL;
}

CVisualClientView::~CVisualClientView()
{
}

BOOL CVisualClientView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	BOOL bPreCreated = CEditView::PreCreateWindow(cs);
	cs.style &= ~(ES_AUTOHSCROLL|WS_HSCROLL);	// Enable word-wrapping

	return bPreCreated;
}


// CVisualClientView diagnostics

#ifdef _DEBUG
void CVisualClientView::AssertValid() const
{
	CEditView::AssertValid();
}

void CVisualClientView::Dump(CDumpContext& dc) const
{
	CEditView::Dump(dc);
}

CVisualClientDoc* CVisualClientView::GetDocument() const // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CVisualClientDoc)));
	return (CVisualClientDoc*)m_pDocument;
}
#endif //_DEBUG


// CVisualClientView message handlers

void CVisualClientView::OnActionsConnect()
{
	if(!m_ptrClient->Executed()) {
		m_ptrClient = new CClient(this);
		if(m_ptrClient->Executed()) {
			MessageBox("You are already connected");
			return;
		} 
		CConnectDlg dlg;
		if(dlg.DoModal() == IDOK) {
			CVisualClientApp* pApp = (CVisualClientApp*)AfxGetApp();
			m_ptrClient->Connect(pApp->m_strHostName.c_str(), pApp->m_nPort);
			if(!m_ptrClient->Connected()) {
				MessageBox(m_ptrClient->Error().c_str());
				return;
			}
			m_ptrClient->Start();
			m_ptrClient->Fire_Login((const char*)dlg.m_strPassword);
			m_tasks.clear();
			m_compilers.clear();
			state = st_wait_for_begining;
		}
	}
}

void CVisualClientView::OnActionsDisconnect()
{
	if(m_ptrClient->Executed()) {
		if(MessageBox("Are you sure to disconnect", NULL, MB_YESNO) == IDYES) {
			m_ptrClient->Stop();
			while(m_ptrClient->Executed()) Sleep(10);
			m_ptrClient->Disconnect();
		}
	}
}

///////////////////////////////////////////////////
// server messages handlers

// Wait For Beginnig
void CVisualClientView::On100() 
{
	const static char mes[] = "  !";
	MessageBox(mes);
	m_ptrLogDlg->AddMessage(mes);
}

// Program Accepted For Testing
void CVisualClientView::On101(const string& taskId) 
{
	const static char mes[] = "     !";
	MessageBox(mes);
	m_ptrLogDlg->AddMessage(mes);
}

// Free Tester Registered
void CVisualClientView::On200() 
{
}

// Logged In
void CVisualClientView::On201() 
{
	const static char mes[] = "   !";
	MessageBox(mes);
	m_ptrLogDlg->AddMessage(mes);
}

// Result of testing
void CVisualClientView::On202(const string& taskId, int result, 
	const string& message, const string& testNumber)
{
	string text = string(": ") + taskId.c_str() + "\n";
	char* res_buf[] = {"OK", "Comppilation error", "Time limit exceeded", 
		"Security violation", "Runtime error", "Presentation error", "Wrong answer"};
	text += string(": ") + res_buf[result];
	switch(result) {
		case 2:
		case 3:
		case 4:
		case 5:
		case 6:
			text += string("   ") + testNumber.c_str();
			break;
	}
	text += "\n";

	MessageBox(text.c_str(), " ");
	m_ptrLogDlg->AddMessage(string(" : ") + text);
}

// Full Rating
void CVisualClientView::On206(const SFullRating& rating) 
{
}

// Part Of Rating
void CVisualClientView::On207(const SPartOfRating& partRating) 
{
}

// Rating Not Changed
void CVisualClientView::On208(const string& timeFrom) 
{
}

// Olympiad Started
void CVisualClientView::On209() 
{
	const static char mes[] = " ";
	MessageBox(mes);
	m_ptrLogDlg->AddMessage(mes);

	state = st_connected;
	m_ptrClient->Fire_GetCompilers();
	m_ptrClient->Fire_GetTasks();
}

// Already Solved
void CVisualClientView::On210(const string& taskId) 
{
	const static char fmes[] = " %s  ";
	char mes[500]; sprintf(mes, fmes, taskId.c_str());
	MessageBox(mes);
	m_ptrLogDlg->AddMessage(mes);
}

// Tasks
void CVisualClientView::On211(const vector< STask >& tasks)
{
	m_tasks = tasks;
	const char mes[] = "  ";
	m_ptrLogDlg->AddMessage(mes);
}

// Compilers
void CVisualClientView::On212(const vector< SCompiler >& compilers)
{
	m_compilers = compilers;
	const char mes[] = "  ";
	m_ptrLogDlg->AddMessage(mes);
}

// Database Credentials 
void CVisualClientView::On213(const string& host, const string& port, const string& db)
{
}

// Olympiad stopped
void CVisualClientView::On214() 
{
	const char mes[] = " ";
	MessageBox(mes);
	m_ptrLogDlg->AddMessage(mes);
	Clear();
}

// <server name> at <host name>
void CVisualClientView::On220(const string& serverName, const string& hostName) 
{
}

// Forbidden
void CVisualClientView::On400() 
{
	const char mes[] = "   -   ";
	MessageBox(mes);
	m_ptrLogDlg->AddMessage(mes);
}

void CVisualClientView::OnBreakConnection()
{
	const char mes[] = "   ";
	MessageBox(mes);
	m_ptrLogDlg->AddMessage(mes);
	Clear();
}
// Method Not Allowed
void CVisualClientView::On401() 
{
	const char mes[] = " :   ";
	MessageBox(mes);
	m_ptrLogDlg->AddMessage(mes);
	Clear();
}

// Tead Disqualified
void CVisualClientView::On402() 
{
	const char mes[] = " ";
	MessageBox(mes);
	m_ptrLogDlg->AddMessage(mes);
	Clear();
}

// Length Required
void CVisualClientView::On403() 
{
	const char mes[] = " :   Content-Length";
	MessageBox(mes);
	m_ptrLogDlg->AddMessage(mes);
}

// Bad Request
void CVisualClientView::On404(const string& message)
{
	char mes[500];
	strcpy(mes, message.empty() ? "Unknown error" : message.c_str());
	MessageBox(mes);
	m_ptrLogDlg->AddMessage(mes);
}

// Code-Tead Disparity
void CVisualClientView::On405() 
{
	const char mes[] = "  ";
	MessageBox(mes);
	m_ptrLogDlg->AddMessage(mes);
	Clear();
}

// Olympiad Currently Running
void CVisualClientView::On407() 
{
}

// Olympiad Is Not In Database
void CVisualClientView::On408() 
{
}

// Olympiad Not Started
void CVisualClientView::On409() 
{
	const char mes[] = "  : o   ";
	MessageBox(mes);
	m_ptrLogDlg->AddMessage(mes);
}

// Internal Server Error
void CVisualClientView::On500(const string& message) 
{
	string mes = string(" : ") + message;
	MessageBox(mes.c_str());
	m_ptrLogDlg->AddMessage(mes);
}

// OLIMPIA Version Not Supported
void CVisualClientView::On502() 
{
	const char mes[] = "   ";
	MessageBox(mes);
	m_ptrLogDlg->AddMessage(mes);
	Clear();
}

// Default
void CVisualClientView::Default(const RRData& rrd)
{
	//MessageBox("Default");
}

void CVisualClientView::Clear()
{
	if(m_ptrClient) {
		m_ptrClient->Stop();
		while(m_ptrClient->Executed()) Sleep(10);
		m_ptrClient->Disconnect();
		m_ptrClient = NULL;
	}

	m_tasks.clear();

	m_compilers.clear();

	while(m_ptrLogDlg->m_listBoxLog.GetCount()) {
		m_ptrLogDlg->m_listBoxLog.DeleteString(0);
	}

	state = st_disconnected;
}

// windows messages handlers

void CVisualClientView::OnActionsTasks()
{
	CTasksDlg(this, m_tasks).DoModal();
}

void CVisualClientView::OnActionsCompilers()
{
	CCompilersDlg(this, m_compilers).DoModal();
}

void CVisualClientView::OnActionsSend()
{
	if(m_ptrClient->Executed()) {
		CSendDlg dlg(this, m_tasks, m_compilers);
		if(dlg.DoModal() == IDOK) {
			CString strProgText;
			GetWindowText(strProgText);
			m_ptrClient->Fire_Task("", m_tasks[dlg.m_nTask].taskNumber, 
				m_tasks[dlg.m_nTask].taskNumber, m_compilers[dlg.m_nCompiler].compilerId,
				(const char*)strProgText);
		}
	}
}

int CVisualClientView::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (__super::OnCreate(lpCreateStruct) == -1)
		return -1;

	SetFont(&m_font, FALSE);

	return 0;
}

void CVisualClientView::OnActionsLog()
{
	m_ptrLogDlg->ShowWindow(SW_SHOW);
}

void CVisualClientView::OnActionsRating()
{
	string path;
	const int buf_size = 500;
	char buf[buf_size];
	GetCurrentDirectory(buf_size, buf);
	path = string(buf) + "\\rv";
	CreateProcess(NULL, "rv\\Rating Viewer.exe", NULL, NULL, FALSE, 0, NULL, path.empty() ? NULL : path.c_str(), &m_si, &m_pi);
}
