// Visual ClientView.h : interface of the CVisualClientView class
//


#pragma once

#include "ptr.h"
#include "sp client.h"
#include "LogDlg.h"


class CVisualClientView : public CEditView, public CClientHandler
{
protected: // create from serialization only
	CVisualClientView();
	DECLARE_DYNCREATE(CVisualClientView)

// Attributes
public:
	CVisualClientDoc* GetDocument() const;

	CountedPtr< CClient > m_ptrClient;
	std::vector< STask > m_tasks;
	std::vector< SCompiler > m_compilers;
	CFont m_font;
	CountedPtr< CLogDlg > m_ptrLogDlg;

	enum t_states { st_connected, st_disconnected, st_wait_for_begining };
	t_states state;

	STARTUPINFO m_si;
	PROCESS_INFORMATION m_pi;
	bool m_bRating;

// Operations
public:

	virtual void OnBreakConnection();

	// Default Handler
	virtual void Default(const RRData& rrd);
	
	// Wait For Beginnig
	virtual void On100();
	
	// Program Accepted For Testing
	virtual void On101(const std::string& taskId);
	
	// Free Tester Registered
	virtual void On200();
	
	// Logged In
	virtual void On201();
	
	// Result of testing
	virtual void On202(const std::string& taskId, int result, 
		const std::string& message, const string& testNumber);

	// Full Rating
	virtual void On206(const SFullRating& rating);

	// Part Of Rating
	virtual void On207(const SPartOfRating& partRating);

	// Rating Not Changed
	virtual void On208(const std::string& timeFrom);

	// Olympiad Started
	virtual void On209();

	// Already Solved
	virtual void On210(const std::string& taskId);

	// Tasks
	virtual void On211(const std::vector< STask >& tasks);

	// Compilers
	virtual void On212(const std::vector< SCompiler >& compilers);

	// Database Credentials 
	virtual void On213(const std::string& host, const std::string& port, const std::string& db);

	// Olympiad stopped
	virtual void On214();

	// <server name> at <host name>
	virtual void On220(const std::string& serverName, const std::string& hostName);
	
	// Forbidden
	virtual void On400();

	// Method Not Allowed
	virtual void On401();

	// Tead Disqualified
	virtual void On402();

	// Length Required
	virtual void On403();

	// Bad Request
	virtual void On404(const std::string& message);

	// Code-Tead Disparity
	virtual void On405();

	// Olympiad Currently Running
	virtual void On407();

	// Olympiad Is Not In Database
	virtual void On408();

	// Olympiad Not Started
	virtual void On409();

	// Internal Server Error
	virtual void On500(const std::string& message);

	// OLIMPIA Version Not Supported
	virtual void On502();

	void Clear();

// Overrides
	public:
virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
protected:

// Implementation
public:
	virtual ~CVisualClientView();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:

// Generated message map functions
protected:
	DECLARE_MESSAGE_MAP()
public:
	afx_msg void OnActionsConnect();
	afx_msg void OnActionsDisconnect();
	afx_msg void OnActionsTasks();
	afx_msg void OnActionsCompilers();
	afx_msg void OnActionsSend();
	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg void OnActionsLog();
	afx_msg void OnActionsRating();
};

#ifndef _DEBUG  // debug version in Visual ClientView.cpp
inline CVisualClientDoc* CVisualClientView::GetDocument() const
   { return reinterpret_cast<CVisualClientDoc*>(m_pDocument); }
#endif

