#pragma once

#include "ptr.h"

#include "pr_socket.h"

#include "buffer.h"
#include "rrdata.h"
#include "protocol.h"
#include "rrparser.h"
#include "handler.h"

class CClientHandler
{
public:
	virtual ~CClientHandler() {}

	virtual void OnBreakConnection() {}

	// Default Handler
	virtual void Default(const RRData& rrd) {}
	
	// Wait For Beginnig
	virtual void On100() {}
	
	// Program Accepted For Testing
	virtual void On101(const std::string& taskId) {}
	
	// Free Tester Registered
	virtual void On200() {}
	
	// Logged In
	virtual void On201() {}
	
	// Result of testing
	virtual void On202(const std::string& taskId, int result, 
		const std::string& message, const string& testNumber)
	{
	}

	// Full Rating
	struct SFullRating {
		struct SCommandRating {
			std::string commandId;
			std::string commandName;
			std::vector< std::string > commandResults;
			int numberDetTasks;
			std::string totalTime;
		};

		std::string lastTimestamp;
		std::vector< SCommandRating > commandResults;
	};
	virtual void On206(const SFullRating& rating) {}

	// Part Of Rating
	struct SPartOfRating {
		struct SPartCommandResult {
			std::string commandId;
			std::string taskNumber;
			int typeResult;
			std::string expTime;
		};

		std::vector< SPartCommandResult > results;
	};
	virtual void On207(const SPartOfRating& partRating) {}

	// Rating Not Changed
	virtual void On208(const std::string& timeFrom) {}

	// Olympiad Started
	virtual void On209() {}

	// Already Solved
	virtual void On210(const std::string& taskId) {}

	// Tasks
	struct STask {
		STask(const std::string _taskNumber, const std::string& _taskName)
			: taskNumber(_taskNumber), taskName(_taskName) {}
		std::string taskNumber;
		std::string taskName;
	};
	virtual void On211(const std::vector< STask >& tasks) {}

	// Compilers
	struct SCompiler {
		SCompiler(const std::string& _compilerId, const std::string& _compilerName)
			: compilerId(_compilerId), compilerName(_compilerName) {}
		std::string compilerId;
		std::string compilerName;
	};
	virtual void On212(const std::vector< SCompiler >& compilers) {}

	// Database Credentials 
	virtual void On213(const std::string& host, const std::string& port, const std::string& db) {}

	// Olympiad stopped
	virtual void On214() {}

	// <server name> at <host name>
	virtual void On220(const std::string& serverName, const std::string& hostName) {}
	
	// Forbidden
	virtual void On400() {}

	// Method Not Allowed
	virtual void On401() {}

	// Tead Disqualified
	virtual void On402() {}

	// Length Required
	virtual void On403() {}

	// Bad Request
	virtual void On404(const std::string& message) {}

	// Code-Tead Disparity
	virtual void On405() {}

	// Olympiad Currently Running
	virtual void On407() {}

	// Olympiad Is Not In Database
	virtual void On408() {}

	// Olympiad Not Started
	virtual void On409() {}

	// Internal Server Error
	virtual void On500(const std::string& message) {}

	// OLIMPIA Version Not Supported
	virtual void On502() {}
};

class CClient 
{
	CClient(const CClient&);
	CClient& operator =(const CClient&);

public:
	static CClientHandler* s_pHandler;

	CClientHandler* m_pHandler;

	bool m_bRunning;
	bool m_bConnected;
	std::string m_strError;

	const string m_strProtocolName;
	int m_nMinor;
	int m_nMajor;
	
	CountedPtr< stdSocket > m_ptrSocket;
	t_handlers m_handlers;
	Handler m_handler;
	CountedPtr< ClientProtocol > m_ptrProtocolReceiver;
	CountedPtr< ClientProtocol > m_ptrProtocolSender;

	HANDLE m_workThread;
	ULONG m_ulThreadId;

	explicit CClient(CClientHandler* pHandler);

	const std::string& Error() const { return m_strError; }
	bool Connect(const char* pszHost, int nPort);
	void Disconnect();
	void Run();
	static DWORD WINAPI WorkThreadProc(LPVOID p);
	void Start();
	void Stop();
	bool Connected() const { return m_bConnected; }
	//bool Executed() const { return m_bRunning; }
	bool Executed() const { return m_workThread == NULL ? false : true; }

	// LOGIN
	void Fire_Login(const std::string& password);
	
	// LOGOUT
	void Fire_Logout();

	// GET-TASKS
	void Fire_GetTasks();

	// GET-COMPILERS
	void Fire_GetCompilers();

	// TASK
	void Fire_Task(const std::string& parameter, const std::string& taskId,
		const std::string& taskNumber, 
		const std::string& compilerId, const std::string& progText);

	// RATING 
	void Fire_Rating(const std::string& parameter);

	// RATING-PART
	void Fire_RatingPart(const std::string& timeFrom);
};
